/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blockentity.machine.iron;

import reborncore.api.IToolDrop;
import reborncore.api.blockentity.InventoryProvider;
import reborncore.client.containerBuilder.IContainerProvider;
import reborncore.client.containerBuilder.builder.BuiltContainer;
import reborncore.client.containerBuilder.builder.ContainerBuilder;
import reborncore.common.crafting.ingredient.RebornIngredient;
import reborncore.common.crafting.RebornRecipe;
import reborncore.common.crafting.ingredient.StackIngredient;
import reborncore.common.registration.RebornRegister;
import reborncore.common.blockentity.MachineBaseBlockEntity;
import reborncore.common.util.RebornInventory;
import techreborn.TechReborn;
import techreborn.init.ModRecipes;
import techreborn.init.TRContent;
import techreborn.init.TRBlockEntities;

import java.util.concurrent.atomic.AtomicInteger;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_2350;
import net.minecraft.class_3866;

@RebornRegister(TechReborn.MOD_ID)
public class IronAlloyFurnaceBlockEntity extends MachineBaseBlockEntity
	implements IToolDrop, InventoryProvider, IContainerProvider {

	public int tickTime;
	public RebornInventory<IronAlloyFurnaceBlockEntity> inventory = new RebornInventory<>(4, "IronAlloyFurnaceBlockEntity", 64, this).withConfiguredAccess();
	public int burnTime;
	public int currentItemBurnTime;
	public int cookTime;
	int input1 = 0;
	int input2 = 1;
	int output = 2;
	int fuel = 3;

	public IronAlloyFurnaceBlockEntity() {
		super(TRBlockEntities.IRON_ALLOY_FURNACE);
	}

	/**
	 * Returns the number of ticks that the supplied fuel item will keep the
	 * furnace burning, or 0 if the item isn't fuel
	 * @param stack Itemstack of fuel
	 * @return Integer Number of ticks
	 */
	public static int getItemBurnTime(class_1799 stack) {
		if (stack.method_7960()) {
			return 0;
		} else {
			return class_3866.method_11196().getOrDefault(stack.method_7909(), 0);
		}
	}

	@Override
	public void method_16896() {
		super.method_16896();
		final boolean flag = this.burnTime > 0;
		boolean flag1 = false;
		if (this.burnTime > 0) {
			--this.burnTime;
		}
		if (!this.field_11863.field_9236) {
			if (this.burnTime != 0 || !inventory.method_5438(this.input1).method_7960()&& !inventory.method_5438(this.fuel).method_7960()) {
				if (this.burnTime == 0 && this.canSmelt()) {
					this.currentItemBurnTime = this.burnTime = IronAlloyFurnaceBlockEntity.getItemBurnTime(inventory.method_5438(this.fuel));
					if (this.burnTime > 0) {
						flag1 = true;
						if (!inventory.method_5438(this.fuel).method_7960()) {
							inventory.shrinkSlot(this.fuel, 1);
						}
					}
				}
				if (this.isBurning() && this.canSmelt()) {
					++this.cookTime;
					if (this.cookTime == 200) {
						this.cookTime = 0;
						this.smeltItem();
						flag1 = true;
					}
				} else {
					this.cookTime = 0;
				}
			}
			if (flag != this.burnTime > 0) {
				flag1 = true;
				// TODO sync on/off
			}
		}
		if (flag1) {
			this.method_5431();
		}
	}

	public boolean hasAllInputs(final RebornRecipe recipeType) {
		if (recipeType == null) {
			return false;
		}
		for (RebornIngredient ingredient : recipeType.getRebornIngredients()) {
			boolean hasItem = false;
			for (int inputslot = 0; inputslot < 2; inputslot++) {
				if (ingredient.test(inventory.method_5438(inputslot))) {
					hasItem = true;
				}
			}
			if (!hasItem)
				return false;
		}
		return true;
	}

	private boolean canSmelt() {
		if (inventory.method_5438(this.input1).method_7960() || inventory.method_5438(this.input2).method_7960()) {
			return false;
		} else {
			class_1799 itemstack = null;
			for (final RebornRecipe recipeType : ModRecipes.ALLOY_SMELTER.getRecipes(field_11863)) {
				if (this.hasAllInputs(recipeType)) {
					itemstack = recipeType.getOutputs().get(0);
					break;
				}
			}

			if (itemstack == null)
				return false;
			if (inventory.method_5438(this.output).method_7960())
				return true;
			if (!inventory.method_5438(this.output).method_7962(itemstack))
				return false;
			final int result = inventory.method_5438(this.output).method_7947() + itemstack.method_7947();
			return result <= inventory.getStackLimit() && result <= inventory.method_5438(this.output).method_7914(); // Forge
			// BugFix:
			// Make
			// it
			// respect
			// stack
			// sizes
			// properly.
		}
	}

	/**
	 * Turn one item from the furnace source stack into the appropriate smelted
	 * item in the furnace result stack
	 */
	public void smeltItem() {
		if (this.canSmelt()) {
			class_1799 itemstack = class_1799.field_8037;
			for (final RebornRecipe recipeType : ModRecipes.ALLOY_SMELTER.getRecipes(field_11863)) {
				if (this.hasAllInputs(recipeType)) {
					itemstack = recipeType.getOutputs().get(0);
					break;
				}
				if (!itemstack.method_7960()) {
					break;
				}
			}

			if (inventory.method_5438(this.output).method_7960()) {
				inventory.method_5447(this.output, itemstack.method_7972());
			} else if (inventory.method_5438(this.output).method_7909() == itemstack.method_7909()) {
				inventory.shrinkSlot(this.output, -itemstack.method_7947());
			}

			for (final RebornRecipe recipeType : ModRecipes.ALLOY_SMELTER.getRecipes(field_11863)) {
				boolean hasAllRecipes = true;
				if (this.hasAllInputs(recipeType)) {

				} else {
					hasAllRecipes = false;
				}
				if (hasAllRecipes) {
					for (RebornIngredient ingredient : recipeType.getRebornIngredients()) {
						for (int inputSlot = 0; inputSlot < 2; inputSlot++) {
							if (ingredient.test(this.inventory.method_5438(inputSlot))) {
								AtomicInteger count = new AtomicInteger(1);
								ingredient.ifType(StackIngredient.class, stackIngredient -> count.set(stackIngredient.getCount()));
								inventory.shrinkSlot(inputSlot, count.get());
								break;
							}
						}
					}
				}
			}

		}
	}

	/**
	 * Furnace isBurning
	 * @return Boolean True if furnace is burning
	 */
	public boolean isBurning() {
		return this.burnTime > 0;
	}

	public int getBurnTimeRemainingScaled(final int scale) {
		if (this.currentItemBurnTime == 0) {
			this.currentItemBurnTime = 200;
		}

		return this.burnTime * scale / this.currentItemBurnTime;
	}

	public int getCookProgressScaled(final int scale) {
		return this.cookTime * scale / 200;
	}

	@Override
	public class_2350 getFacing() {
		return this.getFacingEnum();
	}

	@Override
	public class_1799 getToolDrop(final class_1657 entityPlayer) {
		return TRContent.Machine.IRON_ALLOY_FURNACE.getStack();
	}

	public boolean isComplete() {
		return false;
	}

	@Override
	public RebornInventory<IronAlloyFurnaceBlockEntity> getInventory() {
		return this.inventory;
	}


	public int getBurnTime() {
		return this.burnTime;
	}

	public void setBurnTime(final int burnTime) {
		this.burnTime = burnTime;
	}

	public int getCurrentItemBurnTime() {
		return this.currentItemBurnTime;
	}

	public void setCurrentItemBurnTime(final int currentItemBurnTime) {
		this.currentItemBurnTime = currentItemBurnTime;
	}

	public int getCookTime() {
		return this.cookTime;
	}

	public void setCookTime(final int cookTime) {
		this.cookTime = cookTime;
	}

	@Override
	public BuiltContainer createContainer(int syncID, final class_1657 player) {
		return new ContainerBuilder("alloyfurnace").player(player.field_7514).inventory(8, 84).hotbar(8, 142)
			.addInventory().blockEntity(this)
			.filterSlot(0, 47, 17,
			            stack -> ModRecipes.ALLOY_SMELTER.getRecipes(player.field_6002).stream().anyMatch(recipe -> recipe.getRebornIngredients().get(0).test(stack)))
			.filterSlot(1, 65, 17,
			            stack -> ModRecipes.ALLOY_SMELTER.getRecipes(player.field_6002).stream().anyMatch(recipe -> recipe.getRebornIngredients().get(1).test(stack)))
			.outputSlot(2, 116, 35).fuelSlot(3, 56, 53).syncIntegerValue(this::getBurnTime, this::setBurnTime)
			.syncIntegerValue(this::getCookTime, this::setCookTime)
			.syncIntegerValue(this::getCurrentItemBurnTime, this::setCurrentItemBurnTime).addInventory().create(this, syncID);
	}

	@Override
	public boolean canBeUpgraded() {
		return false;
	}
}
