/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2018 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.blocks.misc;

import net.fabricmc.fabric.api.block.FabricBlockSettings;
import net.minecraft.block.*;
import net.minecraft.class_1268;
import net.minecraft.class_1657;
import net.minecraft.class_1799;
import net.minecraft.class_1937;
import net.minecraft.class_2246;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350;
import net.minecraft.class_2358;
import net.minecraft.class_2410;
import net.minecraft.class_2498;
import net.minecraft.class_2680;
import net.minecraft.class_2689;
import net.minecraft.class_2741;
import net.minecraft.class_2746;
import net.minecraft.class_2753;
import net.minecraft.class_3222;
import net.minecraft.class_3419;
import net.minecraft.class_3481;
import net.minecraft.class_3494;
import net.minecraft.class_3614;
import net.minecraft.class_3620;
import net.minecraft.class_3965;
import reborncore.common.powerSystem.ExternalPowerSystems;
import reborncore.common.powerSystem.ItemPowerManager;
import reborncore.common.util.WorldUtils;
import techreborn.events.TRRecipeHandler;
import techreborn.init.ModSounds;
import techreborn.init.TRContent;
import techreborn.items.tool.ItemTreeTap;
import techreborn.items.tool.basic.ItemElectricTreetap;

import java.util.Random;

/**
 * Created by modmuss50 on 19/02/2016.
 */
public class BlockRubberLog extends class_2410 {

	public static class_2753 SAP_SIDE = class_2741.field_12481;
	public static class_2746 HAS_SAP = class_2746.method_11825("hassap");

	public BlockRubberLog() {
		super(class_3620.field_16017, FabricBlockSettings.of(class_3614.field_15932, class_3620.field_15977).strength(2.0F, 2f).sounds(class_2498.field_11547).ticksRandomly().build());
		this.method_9590(this.method_9564().method_11657(SAP_SIDE, class_2350.field_11043).method_11657(HAS_SAP, false).method_11657(field_11459, class_2350.class_2351.field_11052));
		((class_2358) class_2246.field_10036).method_10189(this, 5, 5);
	}

	@Override
	protected void method_9515(class_2689.class_2690<class_2248, class_2680> builder) {
		super.method_9515(builder);
		builder.method_11667(SAP_SIDE, HAS_SAP);
	}

	@Override
	public boolean method_9525(class_3494<class_2248> tagIn) {
		return tagIn == class_3481.field_15475;
	}

	@Override
	public void method_9576(class_1937 worldIn, class_2338 pos, class_2680 state, class_1657 player) {
		int i = 4;
		int j = i + 1;
		if (worldIn.method_8617(pos.method_10069(-j, -j, -j), pos.method_10069(j, j, j))) {
			for (class_2338 blockpos : class_2338.method_10097(pos.method_10069(-i, -i, -i), pos.method_10069(i, i, i))) {
				class_2680 state1 = worldIn.method_8320(blockpos);
				if (state1.method_11602(class_3481.field_15503)) {
					state1.method_11585(worldIn, blockpos, worldIn.method_8409());
					state1.method_11624(worldIn, blockpos, worldIn.method_8409());
				}
			}
		}
	}

	@Override
	public void method_9588(class_2680 state, class_1937 worldIn, class_2338 pos, Random random) {
		super.method_9588(state, worldIn, pos, random);
		if (state.method_11654(field_11459) != class_2350.class_2351.field_11052) {
			return;
		}
		if (state.method_11654(HAS_SAP)) {
			return;
		}
		if (random.nextInt(50) == 0) {
			class_2350 facing = class_2350.method_10139(random.nextInt(4));
			if (worldIn.method_8320(pos.method_10074()).method_11614() == this
					&& worldIn.method_8320(pos.method_10084()).method_11614() == this) {
				worldIn.method_8501(pos, state.method_11657(HAS_SAP, true).method_11657(SAP_SIDE, facing));
			}
		}
	}

	@Override
	public boolean method_9534(class_2680 state, class_1937 worldIn, class_2338 pos, class_1657 playerIn,
	                                class_1268 hand, class_3965 hitResult) {
		super.method_9534(state, worldIn, pos, playerIn, hand, hitResult);
		class_1799 stack = playerIn.method_5998(class_1268.field_5808);
		if (stack.method_7960()) {
			return false;
		}
		ItemPowerManager capEnergy = null;
		if (stack.method_7909() instanceof ItemElectricTreetap) {
			capEnergy = new ItemPowerManager(stack);
		}
		if ((capEnergy != null && capEnergy.getEnergyStored() > 20) || stack.method_7909() instanceof ItemTreeTap) {
			if (state.method_11654(HAS_SAP) && state.method_11654(SAP_SIDE) == hitResult.method_17780()) {
				worldIn.method_8501(pos, state.method_11657(HAS_SAP, false).method_11657(SAP_SIDE, class_2350.method_10139(0)));
				worldIn.method_8465(null, pos.method_10263(), pos.method_10264(), pos.method_10260(), ModSounds.SAP_EXTRACT, class_3419.field_15245,
						0.6F, 1F);
				if (!worldIn.field_9236) {
					if (capEnergy != null) {
						capEnergy.extractEnergy(20, false);

						ExternalPowerSystems.requestEnergyFromArmor(capEnergy, playerIn);
					} else {
						playerIn.method_5998(class_1268.field_5808).method_7956(1, playerIn, playerEntity -> {});
					}
					if (!playerIn.field_7514.method_7394(TRContent.Parts.SAP.getStack())) {
						WorldUtils.dropItem(TRContent.Parts.SAP.getStack(), worldIn, pos.method_10093(hitResult.method_17780()));
					}
					if (playerIn instanceof class_3222) {
						TRRecipeHandler.unlockTRRecipes((class_3222) playerIn);
					}
				}
				return true;
			}
		}
		return false;
	}

//	@Override
//	public void getDrops(BlockState state, DefaultedList<ItemStack> drops, World world, BlockPos pos, int fortune) {
//		drops.add(new ItemStack(this));
//		if (state.get(HAS_SAP)) {
//			if (new Random().nextInt(4) == 0) {
//				drops.add(TRContent.Parts.SAP.getStack());
//			}
//		}
//	}
}
